@@ -16,7 +16,7 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: nlarn.h 973 2011-05-27 18:13:29Z jaydg $ */
+/* $Id$ */
 
 #ifndef __NLARN_H_
 #define __NLARN_H_
@@ -24,12 +24,12 @@
 #include "game.h"
 
 #define VERSION_MAJOR 0 /* this is the present version # of the program */
-#define VERSION_MINOR 7
+#define VERSION_MINOR 8
 #define VERSION_PATCH 0
 
-/* this allows to add the svn id of the build to the version information */
-#ifndef SVNID
-#define SVNID ""
+/* this allows to add the id of the current commit to the version information */
+#ifndef GITREV
+#define GITREV ""
 #endif
 
 /* the entire game */
