@@ -16,10 +16,10 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: buildings.c 973 2011-05-27 18:13:29Z jaydg $ */
+/* $Id$ */
 
 #include <stdlib.h>
-#include <assert.h>
+#include <glib.h>
 
 #include "container.h"
 #include "display.h"
@@ -97,7 +97,7 @@ int building_bank(player *p)
                               "them. We cannot serve you at this time.  Sorry.\n\n" \
                               "We suggest you go to the LRS office and pay your taxes.";
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     if (Z(p->pos) == 0)
         text = g_string_new(msg_greet);
@@ -152,6 +152,7 @@ int building_bank(player *p)
 
         callback = g_malloc(sizeof(display_inv_callback));
         callback->description = "(s)ell";
+        callback->helpmsg = "Sell the currently selected gem.";
         callback->key = 's';
         callback->inv = &nlarn->store_stock;
         callback->function = &building_item_buy;
@@ -255,7 +256,7 @@ int building_dndstore(player *p)
                                "heart's content. Also be advised that if you " \
                                "break 'em, you pay for 'em.";
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     /* no business if player has outstanding taxes */
     if (p->outstanding_taxes)
@@ -347,7 +348,7 @@ int building_home(player *p)
                            "The doctor thinks that your daughter will recover " \
                            "in a few days.\n\nCongratulations!";
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     /* look for potion of cure dianthroritis in player's inventory */
     if (inv_length_filtered(p->inventory, item_filter_pcd))
@@ -430,6 +431,7 @@ int building_home(player *p)
 
                 callback = g_malloc0(sizeof(display_inv_callback));
                 callback->description = "(d)eposit";
+                callback->helpmsg = "Deposit the selected item in your storage room at home.";
                 callback->key = 'd';
                 callback->inv = &nlarn->player_home;
                 callback->function = &container_item_add;
@@ -453,6 +455,7 @@ int building_home(player *p)
 
                 callback = g_malloc0(sizeof(display_inv_callback));
                 callback->description = "(t)ake";
+                callback->helpmsg = "Take the selected item out of your storage room and put it into your pack.";
                 callback->key = 't';
                 callback->inv = &nlarn->player_home;
                 callback->function = &container_item_unpack;
@@ -481,7 +484,7 @@ int building_lrs(player *p)
     const char msg_taxes[] = "You presently owe %d gp in taxes.";
     const char msg_notax[] = "You do not owe us any taxes.";
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     text = g_string_new(msg_greet);
 
@@ -652,7 +655,7 @@ int building_school(player *p)
         { 10, -1,  "Commission a scroll", NULL }
     };
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     /* courses become more expensive with rising difficulty */
     price = 250 + (game_difficulty(nlarn) * 50);
@@ -784,7 +787,7 @@ int building_tradepost(player *p)
                                "If the items are badly damaged, we will pay " \
                                "only 10% of their new value.";
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     /* no business if player has outstanding taxes */
     if (p->outstanding_taxes)
@@ -798,6 +801,7 @@ int building_tradepost(player *p)
 
     callback = g_malloc(sizeof(display_inv_callback));
     callback->description = "(s)ell";
+    callback->helpmsg = "Sell the selected item to the Trade Post.";
     callback->key = 's';
     callback->inv = &nlarn->store_stock;
     callback->function = &building_item_buy;
@@ -807,6 +811,7 @@ int building_tradepost(player *p)
 
     callback = g_malloc(sizeof(display_inv_callback));
     callback->description = "(i)dentify";
+    callback->helpmsg = "Have the Trade Post's experts identify your item.";
     callback->key = 'i';
     callback->function = &building_item_identify;
     callback->checkfun = &player_item_is_identifiable;
@@ -815,6 +820,7 @@ int building_tradepost(player *p)
 
     callback = g_malloc(sizeof(display_inv_callback));
     callback->description = "(r)epair";
+    callback->helpmsg = "Have the Trade Post's experts revert any damage done to the selected item.";
     callback->key = 'r';
     callback->function = &building_item_repair;
     callback->checkfun = &player_item_is_damaged;
@@ -823,6 +829,7 @@ int building_tradepost(player *p)
 
     callback = g_malloc0(sizeof(display_inv_callback));
     callback->description = "(e)quip";
+    callback->helpmsg = "Equip the selected item.";
     callback->key = 'e';
     callback->function = &player_item_equip;
     callback->checkfun = &player_item_is_equippable;
@@ -830,6 +837,7 @@ int building_tradepost(player *p)
 
     callback = g_malloc(sizeof(display_inv_callback));
     callback->description = "(u)nequip";
+    callback->helpmsg = "Unequip the selected item.";
     callback->key = 'u';
     callback->function = &player_item_unequip_wrapper;
     callback->checkfun = &player_item_is_equipped;
@@ -838,6 +846,7 @@ int building_tradepost(player *p)
 
     callback = g_malloc(sizeof(display_inv_callback));
     callback->description = "(n)ote";
+    callback->helpmsg = "Add a note to the item.";
     callback->key = 'n';
     callback->function = &player_item_notes;
     callback->checkfun = NULL;
@@ -971,7 +980,7 @@ int building_monastery(struct player *p)
         int price;
         int choice;
         char *question;
-        char desc[81];
+        gchar *desc;
 
         if (inv_length_filtered(p->inventory, item_filter_cursed_or_unknown) == 0)
         {
@@ -986,8 +995,7 @@ int building_monastery(struct player *p)
         /* item value for cursed items is reduced by 50% */
         price = (item_price(it) / 5) * (game_difficulty(nlarn) + 1);
 
-        item_describe(it, player_item_identified(p, it), it->count, TRUE,
-                      desc, 80);
+        desc = item_describe(it, player_item_identified(p, it), it->count, TRUE);
 
         question = g_strdup_printf("To remove the curse on %s, we ask you to "
                                    "donate %d gold for our abbey. %s", desc,
@@ -1003,6 +1011,7 @@ int building_monastery(struct player *p)
         }
 
         log_add_entry(nlarn->log, "The monks remove the curse on %s.", desc);
+        g_free(desc);
         item_remove_curse(it);
         building_player_charge(p, price);
         p->stats.gold_spent_donation += price;
@@ -1132,6 +1141,7 @@ static void building_shop(player *p, inventory **inv, const char *title)
 
     callback = g_malloc(sizeof(display_inv_callback));
     callback->description = "(b)uy";
+    callback->helpmsg = "Buy the selected item. If the available quantity exceeds one, you may select the amount you want to purchase.";
     callback->key = 'b';
     callback->inv = inv;
     callback->function = &building_item_sell;
@@ -1228,20 +1238,21 @@ static void building_item_sell(player *p, inventory **inv, item *it)
     guint count = 0;
     gpointer ioid = NULL; /* oid of purchased item */
     char text[81];
-    char name[61];
+    gchar *name;
 
     /* The item the player actually bought. Usually this is the passed item,
      * in the case of item stacks it might be a new item. */
     item *bought_itm = it;
 
-    assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     price = item_price(it);
 
     if (it->count > 1)
     {
-        item_describe(it, TRUE, FALSE, TRUE, name, 40);
+        name = item_describe(it, TRUE, FALSE, TRUE);
         g_snprintf(text, 80, "How many %s do you want to buy?", name);
+        g_free(name);
 
         /* get count */
         count = display_get_count(text, it->count);
@@ -1267,9 +1278,10 @@ static void building_item_sell(player *p, inventory **inv, item *it)
 
         if (!building_player_check(p, price))
         {
-            item_describe(bought_itm, TRUE, (count == 1), TRUE, name, 60);
+            name = item_describe(bought_itm, TRUE, FALSE, TRUE);
             g_snprintf(text, 80, "You cannot afford the %d gold for %s.",
                        price, name);
+            g_free(name);
 
             display_show_message(NULL, text, 0);
 
@@ -1281,17 +1293,16 @@ static void building_item_sell(player *p, inventory **inv, item *it)
     }
     else
     {
-        count = 1;
-        item_describe(it, TRUE, TRUE, TRUE, name, 60);
-        g_snprintf(text, 80, "Do you want to buy %s for %d gold?",
-                   name, price);
+        name = item_describe(it, TRUE, TRUE, TRUE);
+        g_snprintf(text, 80, "Do you want to buy %s for %d gold?", name, price);
+        g_free(name);
 
         if (!display_get_yesno(text, NULL, NULL))
             return;
     }
 
     /* prepare the item description for logging later */
-    item_describe(bought_itm, TRUE, (count == 1), FALSE, name, 60);
+    name = item_describe(bought_itm, TRUE, FALSE, FALSE);
 
     /* try to transfer the item to the player's inventory */
     ioid = bought_itm->oid;
@@ -1323,6 +1334,7 @@ static void building_item_sell(player *p, inventory **inv, item *it)
 
     /* log the event */
     log_add_entry(nlarn->log, "You buy %s. Thank you for your purchase.", name);
+    g_free(name);
 
     /* charge player for this purchase */
     building_player_charge(p, price);
@@ -1330,23 +1342,19 @@ static void building_item_sell(player *p, inventory **inv, item *it)
     player_make_move(p, 2, FALSE, NULL);
 }
 
-static void building_item_identify(player *p, inventory **inv, item *it)
+static void building_item_identify(player *p, inventory **inv __attribute__((unused)), item *it)
 {
     guint price;
-    char name_unknown[61];
-    char name_known[61];
+    gchar *name_unknown;
+    gchar *name_known;
     char message[81];
 
     const char title[] = "Identify item";
 
-    assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
-
-    /* don't need that parameter */
-    inv = NULL;
+    g_assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     price = 50 << game_difficulty(nlarn);
-
-    item_describe(it, player_item_known(p, it), TRUE, TRUE, name_unknown, 60);
+    name_unknown = item_describe(it, player_item_known(p, it), TRUE, TRUE);
 
     if (building_player_check(p, price))
     {
@@ -1357,11 +1365,12 @@ static void building_item_identify(player *p, inventory **inv, item *it)
             player_item_identify(p, NULL, it);
             /* upper case first letter */
             name_unknown[0] = g_ascii_toupper(name_unknown[0]);
-            item_describe(it, player_item_known(p, it), TRUE, FALSE, name_known, 60);
+            name_known = item_describe(it, player_item_known(p, it), TRUE, FALSE);
 
             log_add_entry(nlarn->log, "%s is %s.", name_unknown, name_known);
-            building_player_charge(p, price);
+            g_free(name_known);
 
+            building_player_charge(p, price);
             p->stats.gold_spent_id_repair += price;
             player_make_move(p, 1, FALSE, NULL);
         }
@@ -1371,18 +1380,20 @@ static void building_item_identify(player *p, inventory **inv, item *it)
         g_snprintf(message, 80, "Identifying %s costs %d gold.", name_unknown, price);
         display_show_message(title, message, 0);
     }
+
+    g_free(name_unknown);
 }
 
 static void building_item_repair(player *p, inventory **inv, item *it)
 {
     int damages = 0;
     guint price;
-    char name[61];
+    gchar *name;
     char message[81];
 
     const char title[] = "Repair item";
 
-    assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     /* don't need that parameter */
     inv = NULL;
@@ -1394,7 +1405,7 @@ static void building_item_repair(player *p, inventory **inv, item *it)
 
     price = (50 << game_difficulty(nlarn)) * damages;
 
-    item_describe(it, player_item_known(p, it), TRUE, TRUE, name, 60);
+    name = item_describe(it, player_item_known(p, it), TRUE, TRUE);
 
     if (building_player_check(p, price))
     {
@@ -1419,6 +1430,8 @@ static void building_item_repair(player *p, inventory **inv, item *it)
         g_snprintf(message, 80, "Repairing the %s costs %d gold.", name, price);
         display_show_message(title, message, 0);
     }
+
+    g_free(name);
 }
 
 static void building_item_buy(player *p, inventory **inv, item *it)
@@ -1426,11 +1439,9 @@ static void building_item_buy(player *p, inventory **inv, item *it)
     int price;
     guint count = 0;
     char question[121];
-    char name[61];
+    gchar *name;
 
-    assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
-
-    item_describe(it, player_item_known(p, it), FALSE, FALSE, name, 60);
+    g_assert(p != NULL && it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     price = item_price(it);
 
@@ -1454,10 +1465,12 @@ static void building_item_buy(player *p, inventory **inv, item *it)
 
     if (it->count > 1)
     {
-        item_describe(it, player_item_known(p, it), FALSE, TRUE, name, 60);
+        name = item_describe(it, player_item_known(p, it), FALSE, TRUE);
         g_snprintf(question, 120, "How many %s do you want to sell for %d gold?",
                    name, price);
 
+        g_free(name);
+
         /* get count */
         count = display_get_count(question, it->count);
 
@@ -1475,10 +1488,12 @@ static void building_item_buy(player *p, inventory **inv, item *it)
     else
     {
         count = 1;
-        item_describe(it, player_item_known(p, it), TRUE, TRUE, name, 40);
+        name = item_describe(it, player_item_known(p, it), TRUE, TRUE);
         g_snprintf(question, 120, "Do you want to sell %s for %d gold?",
                    name, price);
 
+        g_free(name);
+
         if (!display_get_yesno(question, NULL, NULL))
             return;
     }
@@ -1491,10 +1506,11 @@ static void building_item_buy(player *p, inventory **inv, item *it)
     guint count_orig = it->count;
     it->count = count;
 
-    item_describe(it, player_item_known(p, it), (count == 1), FALSE, name, 60);
+    name = item_describe(it, player_item_known(p, it), FALSE, FALSE);
     log_add_entry(nlarn->log, "You sell %s. The %d gold %s been transferred to your bank account.",
                   name, price, (price == 1) ? "has" : "have");
 
+    g_free(name);
     it->count = count_orig;
 
     if (it->type == IT_GEM)
