@@ -16,14 +16,12 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: display.c 985 2011-06-10 23:27:30Z jaydg $ */
-
-#include <assert.h>
 #include <curses.h>
+#include <ctype.h>
+#include <glib.h>
 #include <panel.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include "display.h"
 #include "fov.h"
@@ -91,6 +89,8 @@ static int mvwcprintw(WINDOW *win, int defattr, const display_colset *colset,
 
 static int display_get_colval(const display_colset *colset, const char *name);
 
+static void display_inventory_help(GPtrArray *callbacks);
+
 static display_window *display_window_new(int x1, int y1, int width,
         int height, const char *title);
 
@@ -494,111 +494,13 @@ int display_paint_screen(player *p)
     /* wielded weapon */
     if (p->eq_weapon)
     {
-        char wpn[61];
-        const guint available_space = COLS - MAP_MAX_X - 4;
-
-        GString *desc = g_string_new(NULL);
-
-        if (p->eq_weapon->bonus_known)
-        {
-            g_string_append_printf(desc, "%+d ", p->eq_weapon->bonus);
-        }
-
-        const gboolean need_bonus
-        = (p->eq_weapon->burnt || p->eq_weapon->corroded
-           || p->eq_weapon->rusty
-           || (p->eq_weapon->blessed_known
-               && (p->eq_weapon->blessed
-                   || p->eq_weapon->cursed)));
-
-        g_string_append_printf(desc, "%s",
-                               need_bonus ? weapon_short_name(p->eq_weapon)
-                               : weapon_name(p->eq_weapon));
-        g_strlcpy(wpn, desc->str, 60);
-
-        // Add corrosion/curse status in brackets.
-        // Alternatively, convey that information with colours.
-        if (need_bonus && strlen(wpn) < available_space - 4)
-        {
-            GString *bonus = g_string_new(NULL);
-
-            gboolean need_comma = FALSE;
-            if (p->eq_weapon->burnt == 2)
-            {
-                g_string_append_printf(bonus, "v. burnt, ");
-                need_comma = TRUE;
-            }
-
-            if (p->eq_weapon->corroded == 2)
-            {
-                g_string_append_printf(bonus, "%sv. corroded",
-                                       need_comma ? ", " : "");
-                need_comma = TRUE;
-            }
-            if (p->eq_weapon->rusty == 2)
-            {
-                g_string_append_printf(bonus, "%sv. rusty",
-                                       need_comma ? ", " : "");
-                need_comma = TRUE;
-            }
-
-            if (p->eq_weapon->burnt == 1)
-            {
-                g_string_append_printf(bonus, "%sburnt",
-                                       need_comma ? ", " : "");
-                need_comma = TRUE;
-            }
-            if (p->eq_weapon->corroded == 1)
-            {
-                g_string_append_printf(bonus, "%scorroded",
-                                       need_comma ? ", " : "");
-                need_comma = TRUE;
-            }
-            if (p->eq_weapon->rusty == 1)
-            {
-                g_string_append_printf(bonus, "%srusty",
-                                       need_comma ? ", " : "");
-                need_comma = TRUE;
-            }
-
-            if (p->eq_weapon->blessed_known)
-            {
-                if (p->eq_weapon->blessed)
-                {
-                    g_string_append_printf(bonus, "%sblessed",
-                                           need_comma ? ", " : "");
-                }
-                else if (p->eq_weapon->cursed)
-                {
-                    g_string_append_printf(bonus, "%scursed",
-                                           need_comma ? ", " : "");
-                }
-            }
-
-            char info[61];
-            g_strlcpy(info, bonus->str, 60);
-            g_string_append_printf(desc, " (%s)", info);
-
-            g_string_free(bonus, TRUE);
-        }
-        g_strlcpy(wpn, desc->str, 60);
-
-        if (strlen(wpn) > available_space)
-        {
-            if (wpn[available_space - 1] != ' ')
-                wpn[available_space - 1] = '.';
-            wpn[available_space] = '\0';
-        }
-
-        /* free the temporary string */
-        g_string_free(desc, TRUE);
-        mvprintw(7, MAP_MAX_X + 3, "%s", wpn);
+        char *wdesc = weapon_shortdesc(p->eq_weapon, COLS - MAP_MAX_X - 4);
+        mvprintw(7, MAP_MAX_X + 3, "%s", wdesc);
+        g_free(wdesc);
     }
     else
     {
-        attron(DC_WHITE);
         mvprintw(7, MAP_MAX_X + 3, "Unarmed");
-        attroff(DC_WHITE);
     }
     clrtoeol();
 
@@ -649,18 +551,21 @@ int display_paint_screen(player *p)
                     && e->turns < 6)
             {
                 /* fading effects */
-                strv_append_unique(&efdescs, g_strdup_printf("`lightred`%s`end`", desc));
+                gchar *cdesc = g_strdup_printf("`lightred`%s`end`", desc);
+                strv_append_unique(&efdescs, cdesc);
+                g_free(cdesc);
             }
             else if (e->type > ET_LEVITATION)
             {
                 /* negative effects */
-                strv_append_unique(&efdescs, g_strdup_printf("`lightmagenta`%s`end`", desc));
-
+                gchar *cdesc = g_strdup_printf("`lightmagenta`%s`end`", desc);
+                strv_append_unique(&efdescs, cdesc);
+                g_free(cdesc);
             }
             else
             {
                 /* other effects */
-                strv_append_unique(&efdescs, g_strdup(desc));
+                strv_append_unique(&efdescs, desc);
             }
 
             g_free(desc);
@@ -800,9 +705,6 @@ item *display_inventory(const char *title, player *p, inventory **inv,
     char *stitle = NULL;
 
     gboolean keep_running = TRUE;
-
-    /* used for looping over callbacks */
-    display_inv_callback *cb;
     int key;
 
     /* string array used to assemble the window caption
@@ -816,12 +718,11 @@ item *display_inventory(const char *title, player *p, inventory **inv,
     guint curr = 1;
 
     item *it;
-    char item_desc[81];
 
     /* curses attributes */
     int attrs;
 
-    assert(p != NULL && inv != NULL);
+    g_assert(p != NULL && inv != NULL);
 
     /* sort inventory by item type */
     if (show_price)
@@ -911,18 +812,20 @@ item *display_inventory(const char *title, player *p, inventory **inv,
             if (show_price)
             {
                 /* inside shop */
+                gchar *item_desc = item_describe(it, TRUE, FALSE, FALSE);
                 mvwprintw(iwin->window, pos, 1, " %-*s %5d$ ",
-                          width - 11,
-                          item_describe(it, TRUE, FALSE, FALSE, item_desc, 80),
-                          item_price(it));
+                          width - 11, item_desc, item_price(it));
+
+                g_free(item_desc);
             }
             else
             {
+                gchar *item_desc = item_describe(it, player_item_known(p, it), FALSE, FALSE);
                 mvwprintw(iwin->window, pos, 1, " %-*s %c ",
-                          width - 6,
-                          item_describe(it, player_item_known(p, it),
-                                        FALSE, FALSE, item_desc, 80),
+                          width - 6, item_desc,
                           player_item_is_equipped(p, it) ? '*' : ' ');
+
+                g_free(item_desc);
             }
 
             wattroff(iwin->window, attrs);
@@ -958,7 +861,7 @@ item *display_inventory(const char *title, player *p, inventory **inv,
         /* assemble window caption (if callbacks have been defined) */
         for (guint cb_nr = 0; callbacks != NULL && cb_nr < callbacks->len; cb_nr++)
         {
-            cb = g_ptr_array_index(callbacks, cb_nr);
+            display_inv_callback *cb = g_ptr_array_index(callbacks, cb_nr);
 
             /* check if callback is appropriate for this item */
             /* if no checkfunktion is set, always display item */
@@ -974,7 +877,7 @@ item *display_inventory(const char *title, player *p, inventory **inv,
             }
         }
 
-        /* show the item description popup */
+        /* refresh the item description popup */
         if (ipop != NULL)
             display_window_destroy(ipop);
 
@@ -983,6 +886,9 @@ item *display_inventory(const char *title, player *p, inventory **inv,
 
         if (g_strv_length(captions) > 0)
         {
+            /* append "(?) help" to trigger the help popup */
+            strv_append(&captions, "(?) help");
+
             /* update the window's caption with the assembled array of captions */
             display_window_update_caption(iwin, g_strjoinv(" ", captions));
         }
@@ -1093,6 +999,10 @@ item *display_inventory(const char *title, player *p, inventory **inv,
             keep_running = FALSE;
             break;
 
+        case '?':
+            display_inventory_help(callbacks);
+            break;
+
         case KEY_LF:
         case KEY_CR:
 #ifdef PADENTER
@@ -1107,9 +1017,9 @@ item *display_inventory(const char *title, player *p, inventory **inv,
 
         default:
             /* check callback function keys (if defined) */
-            for (guint cb_nr = 1; callbacks != NULL && cb_nr <= callbacks->len; cb_nr++)
+            for (guint cb_nr = 0; callbacks != NULL && cb_nr < callbacks->len; cb_nr++)
             {
-                cb = g_ptr_array_index(callbacks, cb_nr - 1);
+                display_inv_callback *cb = g_ptr_array_index(callbacks, cb_nr);
 
                 if ((cb->key == key) && cb->active)
                 {
@@ -1247,7 +1157,7 @@ spell *display_spell_select(char *title, player *p)
     /* window caption */
     gchar *caption;
 
-    assert(p != NULL);
+    g_assert(p != NULL);
 
     /* buffer for spell code type ahead */
     char *code_buf = g_malloc0(sizeof(char) * 4);
@@ -3082,6 +2992,58 @@ static int mvwcprintw(WINDOW *win, int defattr, const display_colset *colset,
     return count;
 }
 
+static void display_inventory_help(GPtrArray *callbacks)
+{
+    size_t maxlen = 0;
+    GString *help = g_string_new(NULL);
+
+    if (callbacks == NULL || callbacks->len == 0)
+    {
+        /* no callbacks available => select item with ENTER */
+        g_string_append(help, "Select the desired item with ENTER.\n"
+                        "You may abort by pressing the escape key.");
+    }
+    else
+    {
+        /* determine the maximum length of the description */
+        for (guint i = 0; i < callbacks->len; i++)
+        {
+            display_inv_callback *cb = g_ptr_array_index(callbacks, i);
+
+            /* skip this callback function if it is not active */
+            if (!cb->active) continue;
+
+            size_t desclen = strlen(cb->description);
+            if (desclen > maxlen)
+                maxlen = desclen;
+        }
+
+        if (maxlen == 0)
+        {
+            /* no active callbacks */
+            g_string_append(help, "There are no options available for the selected item.");
+        }
+        else
+        {
+            for (guint i = 0; i < callbacks->len; i++)
+            {
+                display_inv_callback *cb = g_ptr_array_index(callbacks, i);
+
+                if (cb->active && cb->helpmsg != NULL)
+                {
+                    g_string_append_printf(help, "`lightgreen`%*s:`end` %s\n",
+                                           (int)maxlen,
+                                           cb->description,
+                                           cb->helpmsg);
+                }
+            }
+        }
+    }
+
+    display_show_message("Help", help->str, maxlen + 2);
+    g_string_free(help, TRUE);
+}
+
 static int display_get_colval(const display_colset *colset, const char *name)
 {
     int colour = 0;
@@ -3162,7 +3124,7 @@ static int display_window_move(display_window *dwin, int key)
 {
     gboolean need_refresh = TRUE;
 
-    assert (dwin != NULL);
+    g_assert (dwin != NULL);
 
     switch (key)
     {
@@ -3216,7 +3178,7 @@ static void display_window_update_title(display_window *dwin, const char *title)
 {
     int attrs; /* curses attributes */
 
-    assert (dwin != NULL && dwin->window != NULL);
+    g_assert (dwin != NULL && dwin->window != NULL);
 
     if (dwin->title)
     {
@@ -3255,7 +3217,7 @@ static void display_window_update_caption(display_window *dwin, char *caption)
 {
     int attrs; /* curses attributes */
 
-    assert (dwin != NULL && dwin->window != NULL);
+    g_assert (dwin != NULL && dwin->window != NULL);
 
     /* repaint line to overwrite previous captions */
     wattron(dwin->window, (attrs = COLOR_PAIR(DCP_BLUE_RED)));
@@ -3280,7 +3242,7 @@ static void display_window_update_arrow_up(display_window *dwin, gboolean on)
 {
     int attrs; /* curses attributes */
 
-    assert (dwin != NULL && dwin->window != NULL);
+    g_assert (dwin != NULL && dwin->window != NULL);
 
     if (on)
     {
@@ -3300,7 +3262,7 @@ static void display_window_update_arrow_down(display_window *dwin, gboolean on)
 {
     int attrs; /* curses attributes */
 
-    assert (dwin != NULL && dwin->window != NULL);
+    g_assert (dwin != NULL && dwin->window != NULL);
 
     if (on)
     {
