@@ -16,9 +16,9 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: items.c 982 2011-06-05 12:20:25Z jaydg $ */
+/* $Id$ */
 
-#include <assert.h>
+#include <glib.h>
 #include <string.h>
 
 #include "amulets.h"
@@ -88,7 +88,7 @@ item *item_new(item_t item_type, int item_id)
     effect *eff = NULL;
     guint loops = 0;
 
-    assert(item_type > IT_NONE && item_type < IT_MAX);
+    g_assert(item_type > IT_NONE && item_type < IT_MAX);
 
     /* has to be zeroed or memcmp will fail */
     nitem = g_malloc0(sizeof(item));
@@ -232,7 +232,7 @@ item *item_new_random(item_t item_type, gboolean finetouch)
     int item_id = 0;
     int min_id = 1, max_id = 0;
 
-    assert(item_type > IT_NONE && item_type < IT_MAX);
+    g_assert(item_type > IT_NONE && item_type < IT_MAX);
 
     max_id = item_max_id(item_type);
 
@@ -271,7 +271,7 @@ item *item_new_by_level(item_t item_type, int num_level)
     int id_min, id_max;
     float variance, id_base, divisor;
 
-    assert (item_type > IT_NONE && item_type < IT_MAX && num_level < MAP_MAX);
+    g_assert (item_type > IT_NONE && item_type < IT_MAX && num_level < MAP_MAX);
 
     /* no amulets above dungeon level 6 */
     if ((item_type == IT_AMULET) && (num_level < 6))
@@ -316,7 +316,7 @@ item *item_new_by_level(item_t item_type, int num_level)
 
 item *item_new_finetouch(item *it)
 {
-    assert(it != NULL);
+    g_assert(it != NULL);
 
     /* maybe the item is blessed or cursed */
     if (it->type == IT_POTION && it->id == PO_WATER)
@@ -370,7 +370,7 @@ item *item_copy(item *original)
 {
     item *nitem;
 
-    assert(original != NULL);
+    g_assert(original != NULL);
 
     /* clone item */
     nitem = g_malloc0(sizeof(item));
@@ -404,7 +404,7 @@ item *item_split(item *original, guint32 count)
 {
     item *nitem;
 
-    assert(original != NULL && count < original->count);
+    g_assert(original != NULL && count < original->count);
 
     nitem = item_copy(original);
 
@@ -416,7 +416,7 @@ item *item_split(item *original, guint32 count)
 
 void item_destroy(item *it)
 {
-    assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     if (it->effects)
     {
@@ -468,6 +468,7 @@ void item_serialize(gpointer oid, gpointer it, gpointer root)
     if (i->cursed == 1) cJSON_AddTrueToObject(ival, "cursed");
     if (i->blessed_known == 1) cJSON_AddTrueToObject(ival, "blessed_known");
     if (i->bonus_known == 1) cJSON_AddTrueToObject(ival, "bonus_known");
+    if (i->fired == 1) cJSON_AddTrueToObject(ival, "fired");
 
     if (i->corroded > 0) cJSON_AddNumberToObject(ival, "corroded", i->corroded);
     if (i->burnt > 0) cJSON_AddNumberToObject(ival, "burnt", i->burnt);
@@ -522,6 +523,9 @@ item *item_deserialize(cJSON *iser, struct game *g)
     obj = cJSON_GetObjectItem(iser, "bonus_known");
     if (obj != NULL) it->bonus_known = obj->valueint;
 
+    obj = cJSON_GetObjectItem(iser, "fired");
+    if (obj != NULL) it->fired = obj->valueint;
+
     obj = cJSON_GetObjectItem(iser, "corroded");
     if (obj != NULL) it->corroded = obj->valueint;
 
@@ -610,13 +614,13 @@ int item_sort(gconstpointer a, gconstpointer b, gpointer data, gboolean force_id
     return order;
 }
 
-char *item_describe(item *it, int known, int singular, int definite, char *str, int str_len)
+gchar *item_describe(item *it, int known, int singular, int definite)
 {
     GString *desc = g_string_new(NULL);
     char *add_info = NULL;
     char **add_infos = strv_new();
 
-    assert((it != NULL) && (it->type > IT_NONE) && (it->type < IT_MAX));
+    g_assert((it != NULL) && (it->type > IT_NONE) && (it->type < IT_MAX));
 
     /* collect additional information */
     if (it->blessed_known)
@@ -806,23 +810,17 @@ char *item_describe(item *it, int known, int singular, int definite, char *str,
         }
     }
 
-    /* copy the assembled string to the destination buffer */
-    g_strlcpy(str, desc->str, str_len);
-
     /* free the additional information */
     g_free(add_info);
 
-    /* free the temporary string */
-    g_string_free(desc, TRUE);
-
-    return(str);
+    return g_string_free(desc, FALSE);
 }
 
 item_material_t item_material(item *it)
 {
     item_material_t material;
 
-    assert (it != NULL);
+    g_assert (it != NULL);
 
     switch (it->type)
     {
@@ -879,7 +877,7 @@ item_material_t item_material(item *it)
 
 guint item_base_price(item *it)
 {
-    assert (it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert (it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     guint price;
 
@@ -938,7 +936,7 @@ guint item_base_price(item *it)
 
 guint item_price(item *it)
 {
-    assert (it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert (it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     guint price = item_base_price(it);
 
@@ -982,7 +980,7 @@ int item_weight(item *it)
 {
     int weight = 0;
 
-    assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     switch (it->type)
     {
@@ -1045,7 +1043,7 @@ int item_weight(item *it)
 
 int item_colour(item *it)
 {
-    assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     switch (it->type)
     {
@@ -1089,7 +1087,7 @@ int item_colour(item *it)
 
 void item_effect_add(item *it, effect *e)
 {
-    assert (it != NULL && it->oid != NULL && e != NULL);
+    g_assert (it != NULL && it->oid != NULL && e != NULL);
 
     /* create list if not existant */
     if (!it->effects)
@@ -1109,7 +1107,7 @@ void item_effect_add(item *it, effect *e)
 
 void item_effect_del(item *it, effect *e)
 {
-    assert (it != NULL && e != NULL);
+    g_assert (it != NULL && e != NULL);
 
     /* del effect from list */
     effect_del(it->effects, e);
@@ -1127,7 +1125,7 @@ void item_effect_del(item *it, effect *e)
 
 int item_bless(item *it)
 {
-    assert (it != NULL && !it->cursed);
+    g_assert (it != NULL && !it->cursed);
 
     if (it->blessed)
         return FALSE;
@@ -1139,7 +1137,7 @@ int item_bless(item *it)
 
 int item_curse(item *it)
 {
-    assert (it != NULL && !it->blessed);
+    g_assert (it != NULL && !it->blessed);
 
     if (it->cursed)
         return FALSE;
@@ -1151,7 +1149,7 @@ int item_curse(item *it)
 
 int item_remove_blessing(item *it)
 {
-    assert (it != NULL && it->blessed);
+    g_assert (it != NULL && it->blessed);
 
     if (!it->blessed)
         return FALSE;
@@ -1164,7 +1162,7 @@ int item_remove_blessing(item *it)
 
 int item_remove_curse(item *it)
 {
-    assert (it != NULL && it->cursed);
+    g_assert (it != NULL && it->cursed);
 
     if (!it->cursed || it->blessed)
         return FALSE;
@@ -1179,9 +1177,8 @@ item *item_enchant(item *it)
 {
     gpointer oid;
     effect *e;
-    char desc[81] = { 0 };
 
-    assert(it != NULL);
+    g_assert(it != NULL);
 
     it->bonus++;
 
@@ -1192,13 +1189,14 @@ item *item_enchant(item *it)
         gboolean bonus_known = it->bonus_known;
         it->bonus_known = FALSE;
 
-        item_describe(it, player_item_known(nlarn->p, it),
-                      (it->count == 1), TRUE, desc, 80);
+        gchar *desc = item_describe(it, player_item_known(nlarn->p, it),
+                                    (it->count == 1), TRUE);
 
         desc[0] = g_ascii_toupper(desc[0]);
         log_add_entry(nlarn->log, "%s vibrate%s strangely.",
                       desc, (it->count == 1) ? "s" : "");
 
+        g_free(desc);
         it->bonus_known = bonus_known;
     }
 
@@ -1228,7 +1226,7 @@ item *item_disenchant(item *it)
     gpointer oid;
     effect *e;
 
-    assert(it != NULL);
+    g_assert(it != NULL);
 
     if (it->bonus <= -3)
     {
@@ -1240,14 +1238,13 @@ item *item_disenchant(item *it)
 
     if (it->bonus == -3)
     {
-        char desc[81] = { 0 };
-        item_describe(it, player_item_known(nlarn->p, it),
-                      (it->count == 1), TRUE, desc, 80);
+        gchar *desc = item_describe(it, player_item_known(nlarn->p, it), FALSE, TRUE);
 
         desc[0] = g_ascii_toupper(desc[0]);
         log_add_entry(nlarn->log, "%s vibrate%s warningly.",
                       desc, (it->count == 1) ? "s" : "");
 
+        g_free(desc);
         it->bonus_known = TRUE;
     }
 
@@ -1284,9 +1281,9 @@ item *item_erode(inventory **inv, item *it, item_erosion_type iet, gboolean visi
 {
     gboolean destroyed = FALSE;
     char *erosion_desc = NULL;
-    char item_desc[61] = { 0 };
+    gchar *item_desc = NULL;
 
-    assert(it != NULL);
+    g_assert(it != NULL);
 
     /* Don't ever destroy the potion of cure dianthroritis.
        This is not currently possible, but add a check in case that changes. */
@@ -1299,8 +1296,9 @@ item *item_erode(inventory **inv, item *it, item_erosion_type iet, gboolean visi
     if (visible)
     {
         /* prepare item description before it has been affected */
-        item_describe(it, player_item_known(nlarn->p, it),
-                      (it->count == 1), TRUE, item_desc, 60);
+        item_desc = item_describe(it, player_item_known(nlarn->p, it),
+                                  (it->count == 1), TRUE);
+
         item_desc[0] = g_ascii_toupper(item_desc[0]);
     }
 
@@ -1311,6 +1309,8 @@ item *item_erode(inventory **inv, item *it, item_erosion_type iet, gboolean visi
         {
             log_add_entry(nlarn->log, "%s resist%s.", item_desc,
                           (it->count == 1) ? "s" : "");
+
+            g_free(item_desc);
             it->blessed_known = TRUE;
         }
         return (it);
@@ -1402,6 +1402,8 @@ item *item_erode(inventory **inv, item *it, item_erosion_type iet, gboolean visi
         it = NULL;
     }
 
+    g_free(item_desc);
+
     return it;
 }
 
@@ -1452,7 +1454,7 @@ int item_obtainable(item_t type, int id)
 
 char *item_detailed_description(item *it, gboolean known, gboolean shop)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
 
     GString *desc = g_string_new("");
 
@@ -1554,7 +1556,7 @@ inventory *inv_new(gconstpointer owner)
 
 void inv_destroy(inventory *inv, gboolean special)
 {
-    assert(inv != NULL);
+    g_assert(inv != NULL);
 
     while (inv_length(inv) > 0)
     {
@@ -1609,7 +1611,7 @@ void inv_callbacks_set(inventory *inv, inv_callback_bool pre_add,
                        inv_callback_void post_add, inv_callback_bool pre_del,
                        inv_callback_void post_del)
 {
-    assert (inv != NULL);
+    g_assert (inv != NULL);
 
     inv-> pre_add  = pre_add;
     inv-> post_add = post_add;
@@ -1619,7 +1621,7 @@ void inv_callbacks_set(inventory *inv, inv_callback_bool pre_add,
 
 int inv_add(inventory **inv, item *it)
 {
-    assert(inv != NULL && it != NULL && it->oid != NULL);
+    g_assert(inv != NULL && it != NULL && it->oid != NULL);
 
     /* create inventory if necessary */
     if (!(*inv))
@@ -1677,7 +1679,7 @@ item *inv_get(inventory *inv, guint idx)
 {
     gpointer oid = NULL;
 
-    assert (inv != NULL && idx < inv->content->len);
+    g_assert (inv != NULL && idx < inv->content->len);
     oid = g_ptr_array_index(inv->content, idx);
 
     return game_item_get(nlarn, oid);
@@ -1687,7 +1689,7 @@ item *inv_del(inventory **inv, guint idx)
 {
     item *itm;
 
-    assert(*inv != NULL && (*inv)->content != NULL && idx < inv_length(*inv));
+    g_assert(*inv != NULL && (*inv)->content != NULL && idx < inv_length(*inv));
 
     itm = inv_get(*inv, idx);
 
@@ -1718,7 +1720,7 @@ item *inv_del(inventory **inv, guint idx)
 
 int inv_del_element(inventory **inv, item *it)
 {
-    assert(*inv != NULL && (*inv)->content != NULL && it != NULL);
+    g_assert(*inv != NULL && (*inv)->content != NULL && it != NULL);
 
     if ((*inv)->pre_del)
     {
@@ -1747,7 +1749,7 @@ int inv_del_element(inventory **inv, item *it)
 
 int inv_del_oid(inventory **inv, gpointer oid)
 {
-    assert(*inv != NULL && (*inv)->content != NULL && oid != NULL);
+    g_assert(*inv != NULL && (*inv)->content != NULL && oid != NULL);
 
     if (!g_ptr_array_remove((*inv)->content, oid))
     {
@@ -1766,7 +1768,7 @@ int inv_del_oid(inventory **inv, gpointer oid)
 
 void inv_erode(inventory **inv, item_erosion_type iet, gboolean visible)
 {
-    assert(inv != NULL);
+    g_assert(inv != NULL);
 
     for (guint idx = 0; idx < inv_length(*inv); idx++)
     {
@@ -1782,7 +1784,7 @@ guint inv_length(inventory *inv)
 
 void inv_sort(inventory *inv, GCompareDataFunc compare_func, gpointer user_data)
 {
-    assert(inv != NULL && inv->content != NULL);
+    g_assert(inv != NULL && inv->content != NULL);
 
     g_ptr_array_sort_with_data(inv->content, compare_func, user_data);
 }
@@ -1869,87 +1871,87 @@ item *inv_get_filtered(inventory *inv, guint idx, int (*ifilter)(item *))
 
 int item_filter_container(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_CONTAINER);
 }
 
 int item_filter_not_container(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type != IT_CONTAINER);
 }
 
 int item_filter_gems(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_GEM);
 }
 
 int item_filter_gold(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_GOLD);
 }
 
 int item_filter_not_gold(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type != IT_GOLD);
 }
 
 int item_filter_potions(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_POTION);
 }
 
 int item_filter_legible(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_SCROLL) || (it->type == IT_BOOK);
 }
 
 int item_filter_unid(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (!player_item_identified(nlarn->p, it));
 }
 
 int item_filter_cursed(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->cursed == TRUE);
 }
 
 int item_filter_cursed_or_unknown(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->cursed == TRUE
             || (it->blessed_known == FALSE && item_is_blessable(it->type)));
 }
 
 int item_filter_nonblessed(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->blessed == FALSE
             || (it->blessed_known == FALSE && item_is_blessable(it->type)));
 }
 
 int item_filter_pcd(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_POTION && it->id == PO_CURE_DIANTHR);
 }
 
 int item_filter_blank_scroll(item *it)
 {
-    assert (it != NULL);
+    g_assert (it != NULL);
     return (it->type == IT_SCROLL && it->id == ST_BLANK);
 }
 
 static const char *item_desc_get(item *it, int known)
 {
-    assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
+    g_assert(it != NULL && it->type > IT_NONE && it->type < IT_MAX);
 
     switch (it->type)
     {
