@@ -16,9 +16,9 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: map.c 977 2011-05-27 20:33:08Z jaydg $ */
+/* $Id$ */
 
-#include <assert.h>
+#include <glib.h>
 #include <stdlib.h>
 
 #include "container.h"
@@ -43,7 +43,7 @@ static int map_validate(map *m);
 static map_path *map_path_new(position start, position goal);
 static map_path_element *map_path_element_new(position pos);
 static int map_step_cost(map *m, map_path_element* element,
-                         map_element_t map_elem, gboolean player);
+                         map_element_t map_elem, gboolean ppath);
 static int map_path_cost(map_path_element* element, position target);
 static map_path_element *map_path_element_in_list(map_path_element* el,
                                                   GPtrArray *list);
@@ -52,7 +52,6 @@ static GPtrArray *map_path_get_neighbours(map *m, position pos,
                                           map_element_t element,
                                           gboolean player);
 
-static gboolean map_monster_destroy(gpointer key, monster *monst, map *m);
 static gboolean map_sphere_destroy(sphere *s, map *m);
 
 const map_tile_data map_tiles[LT_MAX] =
@@ -353,16 +352,24 @@ char *map_dump(map *m, position ppos)
 
 void map_destroy(map *m)
 {
-    assert(m != NULL);
+    g_assert(m != NULL);
 
-    /* destroy monster and spheres on this level */
-    g_hash_table_foreach_remove(nlarn->monsters, (GHRFunc) map_monster_destroy, m);
+    /* destroy spheres on this level */
     g_ptr_array_foreach(nlarn->spheres, (GFunc)map_sphere_destroy, m);
 
-    /* free items */
+    /* destroy items and monsters */
     for (int y = 0; y < MAP_MAX_Y; y++)
         for (int x = 0; x < MAP_MAX_X; x++)
         {
+            if (m->grid[y][x].monster != NULL) {
+                monster *mon = game_monster_get(nlarn, m->grid[y][x].monster);
+
+                /* I wonder why it is possible that a monster ID is stored at
+                 * a position while there is no matching monster registered.
+                 * This seems to occur when called by game_destroy(). */
+                if (mon != NULL) monster_destroy(mon);
+            }
+
             if (m->grid[y][x].ilist != NULL)
                 inv_destroy(m->grid[y][x].ilist, TRUE);
         }
@@ -385,7 +392,7 @@ position map_find_space_in(map *m,
     position pos;
     int count, iteration = 0;
 
-    assert (m != NULL && element > LE_NONE && element < LE_MAX);
+    g_assert (m != NULL && element > LE_NONE && element < LE_MAX);
 
     X(pos) = rand_m_n(where.x1, where.x2);
     Y(pos) = rand_m_n(where.y1, where.y2);
@@ -446,7 +453,7 @@ position map_find_sobject_in(map *m, map_sobject_t sobject, rectangle rect)
 {
     position pos;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     Z(pos) = m->nlevel;
 
@@ -463,7 +470,7 @@ position map_find_sobject(map *m, map_sobject_t sobject)
 {
     position pos;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     Z(pos) = m->nlevel;
 
@@ -481,7 +488,7 @@ gboolean map_pos_validate(map *m, position pos, map_element_t element,
 {
     map_tile *tile;
 
-    assert(m != NULL && element > LE_NONE && element < LE_MAX);
+    g_assert(m != NULL && element > LE_NONE && element < LE_MAX);
 
     /* if the position is invalid it is invalid for the map as well */
     if (!pos_valid(pos))
@@ -650,7 +657,7 @@ int map_pos_is_visible(map *m, position s, position t)
 map_path *map_find_path(map *m, position start, position goal,
                         map_element_t element)
 {
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     map_path *path;
     map_path_element *curr, *next;
@@ -748,7 +755,7 @@ map_path *map_find_path(map *m, position start, position goal,
 
 void map_path_destroy(map_path *path)
 {
-    assert(path != NULL);
+    g_assert(path != NULL);
 
     /* cleanup open list */
     for (guint idx = 0; idx < path->open->len; idx++)
@@ -776,7 +783,7 @@ area *map_get_obstacles(map *m, position center, int radius)
     position pos;
     int x, y;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     if (!pos_valid(center))
     {
@@ -811,7 +818,7 @@ void map_set_tiletype(map *m, area *ar, map_tile_t type, guint8 duration)
     position pos;
     int x, y;
 
-    assert (m != NULL && ar != NULL);
+    g_assert (m != NULL && ar != NULL);
 
     position center;
     X(center) = ar->start_x + ar->size_x / 2;
@@ -853,7 +860,7 @@ void map_set_tiletype(map *m, area *ar, map_tile_t type, guint8 duration)
 
 damage *map_tile_damage(map *m, position pos, gboolean flying)
 {
-    assert (m != NULL && pos_valid(pos));
+    g_assert (m != NULL && pos_valid(pos));
 
     switch (map_tiletype_at(m, pos))
     {
@@ -882,12 +889,11 @@ char *map_pos_examine(position pos)
     map *cm = game_map(nlarn, Z(pos));
     monster *monst;
     item *it;
-    char item_desc[81];
     char *tmp = NULL;
     const char *where;
     GString *desc = g_string_new(NULL);
 
-    assert(pos_valid(pos));
+    g_assert(pos_valid(pos));
 
     if (pos_identical(pos, nlarn->p->pos))
         where = "here";
@@ -947,13 +953,15 @@ char *map_pos_examine(position pos)
             for (guint idx = 0; idx < inv_length(*map_ilist_at(cm, pos)); idx++)
             {
                 it = inv_get(*map_ilist_at(cm, pos), idx);
-                item_describe(it, player_item_known(nlarn->p, it),
-                              FALSE, FALSE, item_desc, 80);
+                gchar *item_desc = item_describe(it, player_item_known(nlarn->p, it),
+                                                 FALSE, FALSE);
 
                 if (idx > 0)
                     g_string_append_printf(items_desc, " and %s", item_desc);
                 else
                     items_desc = g_string_new(item_desc);
+
+                g_free(item_desc);
             }
 
             if (items_desc != NULL)
@@ -969,7 +977,7 @@ char *map_pos_examine(position pos)
 
 monster *map_get_monster_at(map *m, position pos)
 {
-    assert(m != NULL && m->nlevel == Z(pos) && pos_valid(pos));
+    g_assert(m != NULL && m->nlevel == Z(pos) && pos_valid(pos));
 
     gpointer mid = m->grid[Y(pos)][X(pos)].monster;
     return (mid != NULL) ? game_monster_get(nlarn, mid) : NULL;
@@ -980,7 +988,7 @@ void map_fill_with_life(map *m)
     position pos;
     guint new_monster_count;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     new_monster_count = rand_1n(14 + m->nlevel);
 
@@ -1019,7 +1027,7 @@ void map_fill_with_life(map *m)
 
 gboolean map_is_exit_at(map *m, position pos)
 {
-    assert (m != NULL && pos_valid(pos));
+    g_assert (m != NULL && pos_valid(pos));
 
     switch (map_sobject_at(m, pos))
     {
@@ -1043,7 +1051,7 @@ void map_timer(map *m)
     position pos;
     item_erosion_type erosion;
 
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     Z(pos) = m->nlevel;
 
@@ -1102,7 +1110,7 @@ char map_get_door_glyph(map *m, position pos)
 {
     position n, e, s, w;
 
-    assert(m != NULL && pos_valid(pos));
+    g_assert(m != NULL && pos_valid(pos));
 
     n = pos_move(pos, GD_NORTH);
     e = pos_move(pos, GD_EAST);
@@ -1343,7 +1351,7 @@ static void map_fill_with_traps(map *m)
 {
     gboolean trapdoor = FALSE;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     /* Trapdoor cannot be placed in the last dungeon map and the last vulcano map */
     trapdoor = (!is_dungeon_bottom(m->nlevel) && !is_volcano_bottom(m->nlevel));
@@ -1364,7 +1372,7 @@ static void map_make_maze(map *m, int treasure_room)
     rectangle **rooms = NULL;
     gboolean want_monster = FALSE;
 
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     Z(pos) = m->nlevel;
 
@@ -2134,7 +2142,7 @@ static int map_path_cost(map_path_element* element, position target)
 static map_path_element *map_path_element_in_list(map_path_element* el,
                                                   GPtrArray *list)
 {
-    assert(el != NULL && list != NULL);
+    g_assert(el != NULL && list != NULL);
 
     for (guint idx = 0; idx < list->len; idx++)
     {
@@ -2192,21 +2200,6 @@ static GPtrArray *map_path_get_neighbours(map *m, position pos,
     return neighbours;
 }
 
-static gboolean map_monster_destroy(gpointer key __attribute__((unused)),
-                                    monster *monst, map *m)
-{
-    if (Z(monster_pos(monst)) != m->nlevel)
-    {
-        return FALSE;
-    }
-
-    monster_oid_set(monst, 0);
-    monster_destroy(monst);
-
-    return TRUE;
-
-}
-
 static gboolean map_sphere_destroy(sphere *s, map *m)
 {
     if (Z(s->pos) != m->nlevel)
