@@ -16,9 +16,9 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: monsters.c 985 2011-06-10 23:27:30Z jaydg $ */
+/* $Id$ */
 
-#include <assert.h>
+#include <glib.h>
 #include <lua.h>
 #include <lauxlib.h>
 #include <stdlib.h>
@@ -60,6 +60,7 @@ const char *monster_ai_desc[MA_MAX] =
     "standing still",   /* MA_REMAIN */
     "wandering",        /* MA_WANDER */
     "attacking",        /* MA_ATTACK */
+    "puzzled",          /* MA_CONFUSION */
     "serving you",      /* MA_SERVE */
     "doing something boring", /* MA_CIVILIAN */
 };
@@ -106,6 +107,7 @@ static gboolean monster_item_rust(monster *m, struct player *p);
 static gboolean monster_player_rob(monster *m, struct player *p, item_t item_type);
 static position monster_move_wander(monster *m, struct player *p);
 static position monster_move_attack(monster *m, struct player *p);
+static position monster_move_confused(monster *m, struct player *p);
 static position monster_move_flee(monster *m, struct player *p);
 static position monster_move_serve(monster *m, struct player *p);
 static position monster_move_civilian(monster *m, struct player *p);
@@ -115,7 +117,7 @@ static gboolean monster_breath_hit(position pos, const damage_originator *damo,
 
 monster *monster_new(monster_t type, position pos)
 {
-    assert(type > MT_NONE && type < MT_MAX && pos_valid(pos));
+    g_assert(type > MT_NONE && type < MT_MAX && pos_valid(pos));
 
     monster *nmonster;
     int it, icount;     /* item type, item id, item count */
@@ -292,7 +294,7 @@ monster *monster_new(monster_t type, position pos)
 
 monster *monster_new_by_level(position pos)
 {
-    assert(pos_valid(pos));
+    g_assert(pos_valid(pos));
 
     const int mlevel[] = { MT_KOBOLD,           // D1:   5
                             MT_GIANT_ANT,        // D2:  11
@@ -360,7 +362,7 @@ monster *monster_new_by_level(position pos)
 
 void monster_destroy(monster *m)
 {
-    assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
+    g_assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
 
     /* free effects */
     while (m->effects->len > 0)
@@ -377,12 +379,7 @@ void monster_destroy(monster *m)
         inv_destroy(m->inventory, TRUE);
 
     /* unregister monster */
-    if (m->oid != 0)
-    {
-        /* the oid is set to 0 when the monster is destroyed by the
-        g_hash_table_foreach_remove callback */
-        game_monster_unregister(nlarn, m->oid);
-    }
+    game_monster_unregister(nlarn, m->oid);
 
     /* decrement monster count */
     game_map(nlarn, Z(m->pos))->mcount--;
@@ -490,37 +487,31 @@ void monster_deserialize(cJSON *mser, game *g)
 
 int monster_hp_max(monster *m)
 {
-    assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
+    g_assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
     return m->hp_max;
 }
 
 int monster_hp(monster *m)
 {
-    assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
+    g_assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
     return m->hp;
 }
 
 void monster_hp_inc(monster *m, int amount)
 {
-    assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
+    g_assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
     m->hp = min(m->hp + amount, m->hp_max);
 }
 
 gpointer monster_oid(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
     return m->oid;
 }
 
-void monster_oid_set(monster *m, gpointer oid)
-{
-    assert (m != NULL);
-    m->oid = oid;
-}
-
 position monster_pos(monster *m)
 {
-    assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
+    g_assert(m != NULL && m->type > MT_NONE && m->type < MT_MAX);
     return m->pos;
 }
 
@@ -564,7 +555,7 @@ int monster_valid_dest(map *m, position pos, int map_elem)
 
 int monster_pos_set(monster *m, map *mp, position target)
 {
-    assert(m != NULL && mp != NULL && pos_valid(target));
+    g_assert(m != NULL && mp != NULL && pos_valid(target));
 
     if (map_pos_validate(mp, target, monster_map_element(m), FALSE))
     {
@@ -590,19 +581,19 @@ monster_t monster_type(monster *m)
 
 gboolean monster_unknown(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
     return m->unknown;
 }
 
 void monster_unknown_set(monster *m, gboolean what)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
     m->unknown = what;
 }
 
 inventory **monster_inv(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
     return &m->inventory;
 }
 
@@ -617,7 +608,7 @@ static gboolean monster_nearby(monster *m)
 
 gboolean monster_in_sight(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     /* player is blind */
     if (player_effect(nlarn->p, ET_BLINDNESS))
@@ -656,7 +647,7 @@ static const char *get_town_person_name(int value)
 
 monster_action_t monster_action(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     return m->action;
 }
@@ -718,7 +709,7 @@ static int item_is_unique(item *it)
 
 void monster_die(monster *m, struct player *p)
 {
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     /* if the player can see the monster describe the event */
     /* Also give a message for invisible monsters you killed yourself
@@ -801,7 +792,7 @@ void monster_die(monster *m, struct player *p)
 
 void monster_level_enter(monster *m, struct map *l)
 {
-    assert (m != NULL && l != NULL);
+    g_assert (m != NULL && l != NULL);
 
     map_sobject_t source = map_sobject_at(monster_map(m), m->pos);
     map_sobject_t target;
@@ -1017,6 +1008,10 @@ void monster_move(gpointer *oid __attribute__((unused)), monster *m, game *g)
             m_npos = monster_move_attack(m, g->p);
             break;
 
+        case MA_CONFUSION:
+            m_npos = monster_move_confused(m, g->p);
+            break;
+
         case MA_SERVE:
             m_npos = monster_move_serve(m, g->p);
             break;
@@ -1032,10 +1027,14 @@ void monster_move(gpointer *oid __attribute__((unused)), monster *m, game *g)
         }
 
         /* ******** if new position has been found - move the monster ********* */
-        map_sobject_t target_st = map_sobject_at(monster_map(m), m_npos);
-
         if (!pos_identical(m_npos, monster_pos(m)))
         {
+            /* get the monster's current map */
+            map *mmap = monster_map(m);
+
+            /* get stationary object at the monster's target position */
+            map_sobject_t target_st = map_sobject_at(mmap, m_npos);
+
             /* vampires won't step onto mirrors */
             if ((m->type == MT_VAMPIRE) && (target_st == LS_MIRROR))
             {
@@ -1044,35 +1043,83 @@ void monster_move(gpointer *oid __attribute__((unused)), monster *m, game *g)
 
             else if (pos_identical(g->p->pos, m_npos))
             {
-                /* The monster bump into player who is invisible to the monster */
+                /* The monster bumps into the player who is invisible to the
+                   monster. Thus the monster gains knowledge over the player's
+                   current position. */
                 monster_update_player_pos(m, g->p->pos);
 
                 log_add_entry(g->log, "The %s bumps into you.", monster_get_name(m));
             }
 
             /* check for door */
-            else if ((target_st == LS_CLOSEDDOOR)
-                     /* lock out zombies */
-                     && monster_flags(m, MF_HANDS) && monster_int(m) > 3)
+            else if ((target_st == LS_CLOSEDDOOR) && monster_flags(m, MF_HANDS))
             {
-                /* open the door */
-                map_sobject_set(monster_map(m), m_npos, LS_OPENDOOR);
-
-                /* notify the player if the door is visible */
-                if (monster_in_sight(m))
+                /* dim-witted or confused monster are unable to open doors */
+                if (monster_int(m) < 4 || monster_effect_get(m, ET_CONFUSION))
                 {
-                    log_add_entry(g->log, "The %s opens the door.", monster_name(m));
+                    /* notify the player if the door is visible */
+                    if (monster_in_sight(m))
+                    {
+                        log_add_entry(g->log, "The %s bumps into the door.",
+                                      monster_name(m));
+                    }
+                }
+                else
+                {
+                    /* the monster is capable of opening the door */
+                    map_sobject_set(mmap, m_npos, LS_OPENDOOR);
+
+                    /* notify the player if the door is visible */
+                    if (monster_in_sight(m))
+                    {
+                        log_add_entry(g->log, "The %s opens the door.",
+                                      monster_name(m));
+                    }
                 }
             }
 
-            /* move towards player; check for monsters */
-            else if (map_pos_validate(monster_map(m), m_npos,
-                                      monster_map_element(m), FALSE))
+            /* set the monsters new position */
+            else
             {
-                monster_pos_set(m, monster_map(m), m_npos);
+                /* check if the new position is valid for this monster */
+                if (map_pos_validate(mmap, m_npos, monster_map_element(m), FALSE))
+                {
+                    /* the new position is valid -> reposition the monster */
+                    monster_pos_set(m, mmap, m_npos);
+                }
+                else
+                {
+                    /* the new position is invalid */
+                    map_tile_t nle = map_tiletype_at(mmap, m_npos);
+
+                    switch (nle)
+                    {
+                        case LT_TREE:
+                        case LT_WALL:
+                            if (monster_in_sight(m))
+                            {
+                                log_add_entry(g->log, "The %s bumps into %s.",
+                                              monster_name(m), mt_get_desc(nle));
+                            }
+                            break;
+
+                        case LT_LAVA:
+                        case LT_DEEPWATER:
+                            if (monster_in_sight(m)) {
+                                log_add_entry(g->log, "The %s sinks into %s.",
+                                              monster_name(m), mt_get_desc(nle));
+                            }
+                            monster_die(m, g->p);
+                            break;
+
+                        default:
+                            /* just do not move.. */
+                            break;
+                    }
+                }
 
                 /* check for traps */
-                if (map_trap_at(monster_map(m), monster_pos(m)))
+                if (map_trap_at(mmap, monster_pos(m)))
                 {
                     if (!monster_trap_trigger(m))
                         return; /* trap killed the monster */
@@ -1088,7 +1135,7 @@ void monster_move(gpointer *oid __attribute__((unused)), monster *m, game *g)
 
 void monster_polymorph(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     /* make sure mimics never leave the mimicked item behind */
     if (monster_flags(m, MF_MIMIC)
@@ -1154,9 +1201,8 @@ int monster_items_pickup(monster *m)
 
     gboolean pick_up = FALSE;
     item *it;
-    char buf[61] = { 0 };
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     for (guint idx = 0; idx < inv_length(*map_ilist_at(monster_map(m), m->pos)); idx++)
     {
@@ -1197,9 +1243,11 @@ int monster_items_pickup(monster *m)
 
             if (monster_in_sight(m))
             {
-                item_describe(it, player_item_identified(nlarn->p, it),
-                              (it->count == 1), FALSE, buf, 60);
+                gchar *buf = item_describe(it, player_item_identified(nlarn->p, it),
+                                           FALSE, FALSE);
+
                 log_add_entry(nlarn->log, "The %s picks up %s.", monster_name(m), buf);
+                g_free(buf);
             }
 
             inv_del_element(map_ilist_at(monster_map(m), m->pos), it);
@@ -1249,7 +1297,7 @@ attack monster_attack(monster *m, int num)
 {
     attack att = { ATT_NONE, DAM_NONE, 0, 0 };
 
-    assert (m != NULL && num <= monster_attack_count(m));
+    g_assert (m != NULL && num <= monster_attack_count(m));
 
     if (luaN_push_table("monsters", m->type, "attacks"))
     {
@@ -1283,7 +1331,7 @@ attack monster_attack(monster *m, int num)
 
 static int monster_breath_attack(monster *m, player *p, attack att)
 {
-    assert(att.type == ATT_BREATH);
+    g_assert(att.type == ATT_BREATH);
 
     /* FIXME: charm monster is extremely broken. This should be handled totally different */
     if (monster_effect(m, ET_CHARM_MONSTER)
@@ -1351,15 +1399,15 @@ static int modified_attack_amount(int amount, int damage_type)
 void monster_player_attack(monster *m, player *p)
 {
     damage *dam;
+    map *mmap = game_map(nlarn, Z(m->pos));
     attack att = { ATT_NONE, DAM_NONE, 0, 0 };
 
-    assert(m != NULL && p != NULL);
+    g_assert(m != NULL && p != NULL);
 
     /* the player is invisible and the monster bashes into thin air */
     if (!pos_identical(m->player_pos, p->pos))
     {
-        if (!map_is_monster_at(game_map(nlarn, Z(p->pos)), p->pos)
-                && monster_in_sight(m))
+        if (!map_is_monster_at(mmap, p->pos) && monster_in_sight(m))
         {
             log_add_entry(nlarn->log, "The %s bashes into thin air.",
                           monster_name(m));
@@ -1371,7 +1419,8 @@ void monster_player_attack(monster *m, player *p)
     }
 
     /* player is invisible and monster tries to hit player */
-    if (player_effect(p, ET_INVISIBILITY) && !monster_flags(m, MF_INFRAVISION)
+    if (player_effect(p, ET_INVISIBILITY) && !(monster_flags(m, MF_INFRAVISION)
+                                               || monster_effect(m, ET_INFRAVISION))
             && chance(65))
     {
         if (monster_in_sight(m))
@@ -1399,9 +1448,17 @@ void monster_player_attack(monster *m, player *p)
     /* no attack has been found. return to calling function. */
     if (att.type == ATT_NONE) return;
 
+    /* handle breath attacks separately */
+    if (att.type == ATT_BREATH)
+    {
+        monster_breath_attack(m, p, att);
+        return;
+    }
+
     /* generate damage */
     dam = damage_new(att.damage, att.type,
-                     modified_attack_amount(att.base, att.damage), DAMO_MAP, m);
+                     modified_attack_amount(att.base, att.damage),
+                     DAMO_MONSTER, m);
 
     /* deal with random damage (spirit naga) */
     if (dam->type == DAM_RANDOM)
@@ -1410,15 +1467,6 @@ void monster_player_attack(monster *m, player *p)
     if (dam->type == DAM_DEC_RND)
         dam->type = rand_m_n(DAM_DEC_CON, DAM_DEC_RND);
 
-    if (att.type == ATT_BREATH)
-    {
-        /* the damage generated locally is not needed in monster_breath_attack() */
-        damage_free(dam);
-        monster_breath_attack(m, p, att);
-
-        return;
-    }
-
     /* set damage for weapon attacks */
     if ((att.type == ATT_WEAPON) && (m->weapon != NULL))
     {
@@ -1463,8 +1511,7 @@ void monster_player_attack(monster *m, player *p)
         if (monster_player_rob(m, p, (dam->type == DAM_STEAL_GOLD) ? IT_GOLD : IT_ALL))
         {
             /* teleport away */
-            monster_pos_set(m, game_map(nlarn, Z(m->pos)),
-                            map_find_space(game_map(nlarn, Z(m->pos)), LE_MONSTER, FALSE));
+            monster_pos_set(m, mmap, map_find_space(mmap, LE_MONSTER, FALSE));
         }
 
         damage_free(dam);
@@ -1510,7 +1557,7 @@ int monster_player_ranged_attack(monster *m, player *p)
     damage *dam;
     attack att = { ATT_NONE, DAM_NONE, 0, 0 };
 
-    assert(m != NULL && p != NULL);
+    g_assert(m != NULL && p != NULL);
 
     /* choose a random attack type */
     att = monster_attack(m, rand_1n(monster_attack_count(m) + 1));
@@ -1529,7 +1576,7 @@ int monster_player_ranged_attack(monster *m, player *p)
         else
         {
             dam = damage_new(att.damage, att.type, att.base + game_difficulty(nlarn),
-                             DAMO_MAP, m);
+                             DAMO_MONSTER, m);
 
             player_damage_take(p, dam, PD_MONSTER, m->type);
         }
@@ -1546,7 +1593,7 @@ monster *monster_damage_take(monster *m, damage *dam)
     struct player *p = NULL;
     int hp_orig;
 
-    assert(m != NULL && dam != NULL);
+    g_assert(m != NULL && dam != NULL);
 
     if (dam->dam_origin.ot == DAMO_PLAYER)
         p = (player *)dam->dam_origin.originator;
@@ -1665,11 +1712,6 @@ gboolean monster_update_action(monster *m, monster_action_t override)
     gboolean low_hp;
     gboolean smart;
 
-    /* FIXME: should include difficulty here */
-    mtime  = monster_int(m) + 25;
-    low_hp = (m->hp < (monster_hp_max(m) / 4 ));
-    smart  = (monster_int(m) > 4);
-
     if (override > MA_NONE)
     {
         /* set the monster's action to the requested value */
@@ -1684,17 +1726,33 @@ gboolean monster_update_action(monster *m, monster_action_t override)
         return TRUE;
     }
 
-    if (m->action == MA_SERVE)
+    /* handle some easy action updates before the more difficult decisions */
+    switch (m->action)
     {
-        /* once servant, forever servant */
-        return FALSE;
+        case MA_SERVE:
+            /* once servant, forever servant */
+            return FALSE;
+            break;
+
+        case MA_CIVILIAN:
+            /* town people never change their behaviour */
+            return FALSE;
+            break;
+
+        case MA_CONFUSION:
+            /* confusion is removed by monster_effect_del() */
+            return FALSE;
+            break;
+
+        default:
+            /* countinue to evaluate... */
+            break;
     }
 
-    if (m->action == MA_CIVILIAN)
-    {
-        /* town people never change their behaviour */
-        return FALSE;
-    }
+    /* FIXME: should include difficulty here */
+    mtime  = monster_int(m) + 25;
+    low_hp = (m->hp < (monster_hp_max(m) / 4 ));
+    smart  = (monster_int(m) > 4);
 
     if (monster_flags(m, MF_MIMIC) && m->unknown)
     {
@@ -1737,7 +1795,7 @@ gboolean monster_update_action(monster *m, monster_action_t override)
 
 void monster_update_player_pos(monster *m, position ppos)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     m->player_pos = ppos;
     m->lastseen = 1;
@@ -1751,7 +1809,7 @@ gboolean monster_regenerate(monster *m, time_t gtime, int difficulty)
     /* temporary var for effect */
     effect *e;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     /* modify frequency by difficulty: more regeneration, less poison */
     frequency = difficulty << 1;
@@ -1785,7 +1843,7 @@ gboolean monster_regenerate(monster *m, time_t gtime, int difficulty)
 
 item *get_mimic_item(monster *m)
 {
-    assert(m && monster_flags(m, MF_MIMIC));
+    g_assert(m && monster_flags(m, MF_MIMIC));
 
     /* polymorphed mimics may not pose as items */
     if (inv_length(m->inventory) > 0)
@@ -1800,7 +1858,7 @@ char *monster_desc(monster *m)
     GString *desc;
     char *injury, *effects = NULL;
 
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     desc = g_string_new(NULL);
 
@@ -1808,12 +1866,11 @@ char *monster_desc(monster *m)
     if (monster_unknown(m) && inv_length(m->inventory) > 0)
     {
         item *it = get_mimic_item(m);
-        char item_desc[81] = { 0 };
-
-        item_describe(it, player_item_known(nlarn->p, it), (it->count == 1),
-                      FALSE, item_desc, 80);
+        gchar *item_desc= item_describe(it, player_item_known(nlarn->p, it),
+                                        FALSE, FALSE);
 
         g_string_append_printf(desc, "You see %s there", item_desc);
+        g_free(item_desc);
 
         return g_string_free(desc, FALSE);
     }
@@ -1832,18 +1889,25 @@ char *monster_desc(monster *m)
     else
         injury = "critically injured";
 
-    GString *hp_string = g_string_new("");
+    g_string_append_printf(desc, "%s %s, %s %s", a_an(injury), injury,
+                           monster_ai_desc[m->action], monster_get_name(m));
+
     if (game_wizardmode(nlarn))
     {
-        g_string_append_printf(hp_string, " (%d/%d hp)",
-                               m->hp, m->hp_max);
+        /* show monster's hp and max hp in wizard mode */
+        g_string_append_printf(desc, " (%d/%d hp)", m->hp, m->hp_max);
     }
-    g_string_append_printf(desc, "%s %s %s%s, %s", a_an(injury),
-                           injury, monster_get_name(m),
-                           hp_string->str,
-                           monster_ai_desc[m->action]);
 
-    g_string_free(hp_string, TRUE);
+    if (m->weapon != NULL)
+    {
+        /* describe the weapon the monster wields */
+        item *weapon = game_item_get(nlarn, m->weapon);
+        gchar *weapon_desc = item_describe(weapon, player_item_known(nlarn->p, weapon),
+                                           TRUE, FALSE);
+
+        g_string_append_printf(desc, ", armed with %s", weapon_desc);
+        g_free(weapon_desc);
+    }
 
     /* add effect description */
     if (m->effects->len > 0)
@@ -1873,7 +1937,7 @@ char *monster_desc(monster *m)
 
 char monster_glyph(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     if (m->unknown && inv_length(m->inventory) > 0)
     {
@@ -1888,7 +1952,7 @@ char monster_glyph(monster *m)
 
 int monster_color(monster *m)
 {
-    assert (m != NULL);
+    g_assert (m != NULL);
 
     if (m->unknown && inv_length(m->inventory) > 0)
     {
@@ -1906,7 +1970,7 @@ void monster_genocide(monster_t monster_id)
     GList *mlist;
     monster *monst;
 
-    assert(monster_id > MT_NONE && monster_id < MT_MAX);
+    g_assert(monster_id > MT_NONE && monster_id < MT_MAX);
 
     nlarn->monster_genocided[monster_id] = TRUE;
     mlist = g_hash_table_get_values(nlarn->monsters);
@@ -1931,13 +1995,13 @@ void monster_genocide(monster_t monster_id)
 
 int monster_is_genocided(monster_t monster_id)
 {
-    assert(monster_id > MT_NONE && monster_id < MT_MAX);
+    g_assert(monster_id > MT_NONE && monster_id < MT_MAX);
     return nlarn->monster_genocided[monster_id];
 }
 
 effect *monster_effect_add(monster *m, effect *e)
 {
-    assert(m != NULL && e != NULL);
+    g_assert(m != NULL && e != NULL);
     gboolean vis_effect = FALSE;
 
     if (e->type == ET_SLEEP && monster_flags(m, MF_RES_SLEEP))
@@ -1958,6 +2022,12 @@ effect *monster_effect_add(monster *m, effect *e)
         effect_destroy(e);
         e = NULL;
     }
+    else if (e->type == ET_CONFUSION && monster_flags(m, MF_RES_CONF))
+    {
+        /* the monster is resistant against confusion */
+        effect_destroy(e);
+        e = NULL;
+    }
 
     /* one time effects */
     if (e && e->turns == 1)
@@ -1992,6 +2062,11 @@ effect *monster_effect_add(monster *m, effect *e)
     {
         /* multi-turn effects */
         e = effect_add(m->effects, e);
+
+        /* if it's confusion, set the monster's "AI" accordingly */
+        if (e && e->type == ET_CONFUSION) {
+            monster_update_action(m, MA_CONFUSION);
+        }
     }
 
     /* show message if monster is visible */
@@ -2017,7 +2092,7 @@ int monster_effect_del(monster *m, effect *e)
 {
     int result;
 
-    assert(m != NULL && e != NULL);
+    g_assert(m != NULL && e != NULL);
 
     /* log info if the player can see the monster */
     if (monster_in_sight(m) && effect_get_msg_m_stop(e))
@@ -2027,6 +2102,11 @@ int monster_effect_del(monster *m, effect *e)
 
     if ((result = effect_del(m->effects, e)))
     {
+        /* if confusion is finished, set the ai back to the default */
+        if ((e->type) == ET_CONFUSION) {
+            monster_update_action(m, MA_WANDER);
+        }
+
         effect_destroy(e);
     }
 
@@ -2035,13 +2115,13 @@ int monster_effect_del(monster *m, effect *e)
 
 effect *monster_effect_get(monster *m , effect_t type)
 {
-    assert(m != NULL && type < ET_MAX);
+    g_assert(m != NULL && type < ET_MAX);
     return effect_get(m->effects, type);
 }
 
 int monster_effect(monster *m, effect_t type)
 {
-    assert(m != NULL && type < ET_MAX);
+    g_assert(m != NULL && type < ET_MAX);
     return effect_query(m->effects, type);
 }
 
@@ -2049,7 +2129,7 @@ void monster_effects_expire(monster *m)
 {
     guint idx = 0;
 
-    assert(m != NULL);
+    g_assert(m != NULL);
 
     while (idx < m->effects->len)
     {
@@ -2101,6 +2181,7 @@ static gboolean monster_player_visible(monster *m)
     if (pos_distance(monster_pos(m), nlarn->p->pos) > monster_visrange)
         return FALSE;
 
+    /* check if the player is invisible and if the monster has infravision */
     if (player_effect(nlarn->p, ET_INVISIBILITY)
         && !(monster_flags(m, MF_INFRAVISION) || monster_effect(m, ET_INFRAVISION)))
         return FALSE;
@@ -2157,19 +2238,17 @@ static item *monster_weapon_select(monster *m)
 
 static void monster_weapon_wield(monster *m, item *weapon)
 {
-    char buf[61] = { 0 };
-
     /* FIXME: time management */
     m->weapon = weapon->oid;
 
     /* show message if monster is visible */
     if (monster_in_sight(m))
     {
-        item_describe(weapon, player_item_identified(nlarn->p, weapon),
-                      TRUE, FALSE, buf, 60);
+        gchar *buf = item_describe(weapon, player_item_identified(nlarn->p, weapon),
+                                   TRUE, FALSE);
 
-        log_add_entry(nlarn->log, "The %s wields %s.",
-                      monster_name(m), buf);
+        log_add_entry(nlarn->log, "The %s wields %s.", monster_name(m), buf);
+        g_free(buf);
     }
 }
 
@@ -2177,7 +2256,7 @@ static gboolean monster_item_disenchant(monster *m, struct player *p)
 {
     item *it;
 
-    assert (m != NULL && p != NULL);
+    g_assert (m != NULL && p != NULL);
 
     /* disenchant random item */
     if (!inv_length(p->inventory))
@@ -2199,15 +2278,15 @@ static gboolean monster_item_disenchant(monster *m, struct player *p)
     // Blessed items have a 50% chance of resisting the disenchantment.
     if (it->blessed && chance(50))
     {
-        char desc[81] = { 0 };
-        item_describe(it, player_item_known(nlarn->p, it),
-                      (it->count == 1), TRUE, desc, 80);
+        gchar *desc = item_describe(it, player_item_known(nlarn->p, it),
+                                    (it->count == 1), TRUE);
 
         desc[0] = g_ascii_toupper(desc[0]);
         log_add_entry(nlarn->log, "%s resist%s the attack.",
                       desc, (it->count == 1) ? "s" : "");
 
         it->blessed_known = TRUE;
+        g_free(desc);
         return TRUE;
     }
     log_add_entry(nlarn->log, "You feel a sense of loss.");
@@ -2238,7 +2317,7 @@ static gboolean monster_item_rust(monster *m __attribute__((unused)), struct pla
 {
     item **it;
 
-    assert(m != NULL && p != NULL);
+    g_assert(m != NULL && p != NULL);
 
     /* get a random piece of armour to damage */
     if ((it = player_get_random_armour(p, FALSE)))
@@ -2255,10 +2334,9 @@ static gboolean monster_item_rust(monster *m __attribute__((unused)), struct pla
 
 static gboolean monster_player_rob(monster *m, struct player *p, item_t item_type)
 {
-    guint player_gold = 0;
     item *it = NULL;
 
-    assert (m != NULL && p != NULL);
+    g_assert (m != NULL && p != NULL);
 
     /* if player has a device of no theft abort the theft */
     if (player_effect(p, ET_NOTHEFT))
@@ -2267,7 +2345,10 @@ static gboolean monster_player_rob(monster *m, struct player *p, item_t item_typ
     /* Leprechaun robs only gold */
     if (item_type == IT_GOLD)
     {
-        if ((player_gold = player_get_gold(p)))
+        /* get amount of gold pieces carried by the player */
+        guint player_gold = player_get_gold(p);
+
+        if (player_gold > 0)
         {
             it = item_new(IT_GOLD, rand_1n(1 + (player_gold >> 1)));
             player_remove_gold(p, it->count);
@@ -2303,12 +2384,10 @@ static gboolean monster_player_rob(monster *m, struct player *p, item_t item_typ
     {
         if (inv_length(p->inventory))
         {
-            char buf[61];
             gboolean was_equipped = FALSE;
 
             it = inv_get(p->inventory, rand_0n(inv_length(p->inventory)));
-            item_describe(it, player_item_known(p, it), (it->count == 1),
-                          FALSE, buf, 60);
+            gchar *buf = item_describe(it, player_item_known(p, it), FALSE, FALSE);
 
             if ((was_equipped = player_item_is_equipped(p, it)))
             {
@@ -2317,7 +2396,9 @@ static gboolean monster_player_rob(monster *m, struct player *p, item_t item_typ
                     /* cursed items can't be stolen.. */
                     log_add_entry(nlarn->log, "The %s tries to steal %s but fails.",
                                   monster_get_name(m), buf);
+
                     it->blessed_known = TRUE;
+                    g_free(buf);
 
                     /* return true as there are things to steal */
                     return TRUE;
@@ -2332,8 +2413,8 @@ static gboolean monster_player_rob(monster *m, struct player *p, item_t item_typ
             {
                 /* the player has multiple items. Steal only one. */
                 it = item_split(it, rand_1n(it->count));
-                item_describe(it, player_item_known(p, it), (it->count == 1),
-                              FALSE, buf, 60);
+                g_free(buf);
+                buf = item_describe(it, player_item_known(p, it), FALSE, FALSE);
             }
             else
             {
@@ -2351,6 +2432,8 @@ static gboolean monster_player_rob(monster *m, struct player *p, item_t item_typ
                 log_add_entry(nlarn->log, "The %s picks your pocket and steals %s.",
                               monster_get_name(m), buf);
             }
+
+            g_free(buf);
         }
     }
 
@@ -2376,10 +2459,7 @@ static char *monsters_get_fortune(char *fortune_file)
 
     if (!fortunes)
     {
-
         /* read in the fortunes */
-
-        size_t len = 0;
         char buffer[80];
         char *tmp = 0;
         FILE *fortune_fd;
@@ -2400,7 +2480,7 @@ static char *monsters_get_fortune(char *fortune_file)
             while((fgets(buffer, 79, fortune_fd)))
             {
                 /* replace EOL with \0 */
-                len = (size_t)(strchr(buffer, '\n') - (char *)&buffer);
+                size_t len = (size_t)(strchr(buffer, '\n') - (char *)&buffer);
                 buffer[len] = '\0';
 
                 /* keep the line */
@@ -2520,6 +2600,13 @@ static position monster_move_attack(monster *m, struct player *p)
 
 }
 
+static position monster_move_confused(monster *m,
+                                      __attribute__ ((unused)) struct player *p)
+{
+    /* as the monster is confused, choos a random movement direction */
+    return pos_move(monster_pos(m), rand_1n(GD_MAX));
+}
+
 static position monster_move_flee(monster *m, struct player *p)
 {
     int dist = 0;
@@ -2558,7 +2645,9 @@ static position monster_move_serve(monster *m, struct player *p)
 
     /* calculate the monster's fov */
     /* the monster gets a fov radius of 6 for now*/
-    fov_calculate(m->fov, monster_map(m), m->pos, 6, monster_flags(m, MF_INFRAVISION));
+    fov_calculate(m->fov, monster_map(m), m->pos, 6,
+                  monster_flags(m, MF_INFRAVISION)
+                  || monster_effect(m, ET_INFRAVISION));
 
     /* a good servant always knows the masters position */
     if (pos_distance(monster_pos(m), p->pos) > 5)
