@@ -16,9 +16,9 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: sobjects.c 973 2011-05-27 18:13:29Z jaydg $ */
+/* $Id$ */
 
-#include <assert.h>
+#include <glib.h>
 
 #include "display.h"
 #include "game.h"
@@ -36,7 +36,7 @@ int player_altar_desecrate(player *p)
     effect *e = NULL;
     map *current;
 
-    assert (p != NULL);
+    g_assert (p != NULL);
 
     current = game_map(nlarn, Z(p->pos));
 
@@ -84,7 +84,7 @@ int player_altar_pray(player *p)
     item **armour = NULL;
     map *current;
 
-    assert (p != NULL);
+    g_assert (p != NULL);
 
     current = game_map(nlarn, Z(p->pos));
 
@@ -434,18 +434,18 @@ int player_door_open(player *p, int dir)
     /* position used to interact with stationaries */
     position pos;
 
-    /* a counter and another one */
-    int count, num;
-
     /* the current map */
     map *pmap = game_map(nlarn, Z(p->pos));
 
     if (dir == GD_NONE)
     {
+        /* number of valid directions for actions */
+        int count = 0;
+
         /* possible directions of actions */
         int *dirs = map_get_surrounding(pmap, p->pos, LS_CLOSEDDOOR);
 
-        for (count = 0, num = 1; num < GD_MAX; num++)
+        for (int num = 1; num < GD_MAX; num++)
         {
             if (dirs[num])
             {
@@ -502,11 +502,9 @@ int player_fountain_drink(player *p)
     effect *e = NULL;
 
     int fntchange = 0;
-    int amount = 0;
-    int et = ET_NONE;
     map *pmap = game_map(nlarn, Z(p->pos));
 
-    assert (p != NULL);
+    g_assert (p != NULL);
 
     if (map_sobject_at(pmap, p->pos) == LS_DEADFOUNTAIN)
     {
@@ -550,6 +548,9 @@ int player_fountain_drink(player *p)
 
     if (fntchange != 0)
     {
+        int amount = 0;
+        effect_t et = ET_NONE;
+
         switch (rand_1n(9))
         {
         case 1:
@@ -660,7 +661,7 @@ int player_fountain_wash(player *p)
 {
     map *pmap = game_map(nlarn, Z(p->pos));
 
-    assert (p != NULL);
+    g_assert (p != NULL);
 
     if (map_sobject_at(pmap, p->pos) == LS_DEADFOUNTAIN)
     {
@@ -852,7 +853,7 @@ int player_throne_pillage(player *p)
     /* type of object at player's position */
     map_sobject_t ms = map_sobject_at(pmap, p->pos);
 
-    assert (p != NULL);
+    g_assert (p != NULL);
 
     if ((ms != LS_THRONE) && (ms != LS_THRONE2) && (ms != LS_DEADTHRONE))
     {
@@ -911,7 +912,7 @@ int player_throne_sit(player *p)
     map *pmap = game_map(nlarn, Z(p->pos));
     map_sobject_t st = map_sobject_at(pmap, p->pos);
 
-    assert (p != NULL);
+    g_assert (p != NULL);
 
     if ((st != LS_THRONE) && (st != LS_THRONE2) && (st != LS_DEADTHRONE))
     {
